@@ -1,3 +1,4 @@
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE NamedFieldPuns #-}
@@ -59,9 +60,7 @@ import qualified Data.Map.Strict as Map
 import qualified GHC
 import GHC hiding (EpaComment)
 import GHC.Base (NonEmpty(..))
-import GHC.Data.Bag
 import GHC.Data.FastString
-import qualified GHC.Data.Strict as Strict
 import GHC.Driver.Ppr
 import GHC.Hs.Dump
 import GHC.Parser.Lexer (allocateComments)
@@ -69,7 +68,6 @@ import GHC.Types.Name
 import GHC.Types.Name.Reader
 import GHC.Types.SrcLoc
 
-import Language.Haskell.GHC.ExactPrint.Lookup
 import Language.Haskell.GHC.ExactPrint.Types
 
 import Debug.Trace
@@ -169,17 +167,10 @@ undelta (l,_) (DifferentLine dl dc) (LayoutStartCol co) = (fl,fc)
     fl = l + dl
     fc = co + dc
 
-undeltaSpan :: RealSrcSpan -> AnnKeywordId -> DeltaPos -> AddEpAnn
-undeltaSpan anc kw dp = AddEpAnn kw (EpaSpan (RealSrcSpan sp Strict.Nothing))
-  where
-    (l,c) = undelta (ss2pos anc) dp (LayoutStartCol 0)
-    len = length (keywordToString kw)
-    sp = range2rs ((l,c),(l,c+len))
-
 -- ---------------------------------------------------------------------
 
 adjustDeltaForOffset :: LayoutStartCol -> DeltaPos -> DeltaPos
-adjustDeltaForOffset _colOffset                      dp@(SameLine _) = dp
+adjustDeltaForOffset _colOffset              dp@(SameLine _) = dp
 adjustDeltaForOffset (LayoutStartCol colOffset) (DifferentLine l c)
   = DifferentLine l (c - colOffset)
 
@@ -233,15 +224,18 @@ isPointSrcSpan ss = spanLength ss == 0
 -- `MovedAnchor` operation based on the original location, only if it
 -- does not already have one.
 commentOrigDelta :: LEpaComment -> LEpaComment
-commentOrigDelta (L (EpaSpan (RealSrcSpan la _)) (GHC.EpaComment t pp))
-  = (L (EpaDelta dp NoComments) (GHC.EpaComment t pp))
-                  `debug` ("commentOrigDelta: (la, pp, r,c, dp)=" ++ showAst (la, pp, r,c, dp))
+commentOrigDelta (L (EpaSpan ss@(RealSrcSpan la _)) (GHC.EpaComment t pp))
+  = (L (EpaDelta ss dp' NoComments) (GHC.EpaComment t pp))
+                  `debug` ("commentOrigDelta: (la, pp, r,c, dp, dp')=" ++ showAst (la, pp, r,c, dp, dp'))
   where
         (r,c) = ss2posEnd pp
 
         dp = if r == 0
                then (ss2delta (r,c+1) la)
                else (ss2delta (r,c)   la)
+        dp' = case dp of
+            SameLine _ -> dp
+            DifferentLine l cc -> DifferentLine l (cc - 1)
 commentOrigDelta c = c
 
 origDelta :: RealSrcSpan -> RealSrcSpan -> DeltaPos
@@ -275,7 +269,7 @@ insertCppComments (L l p) cs0 = insertRemainingCppComments (L l p2) remaining
     addCommentsListItem :: EpAnn AnnListItem -> State [LEpaComment] (EpAnn AnnListItem)
     addCommentsListItem = addComments
 
-    addCommentsList :: EpAnn AnnList -> State [LEpaComment] (EpAnn AnnList)
+    addCommentsList :: EpAnn (AnnList ()) -> State [LEpaComment] (EpAnn (AnnList ()))
     addCommentsList = addComments
 
     addCommentsGrhs :: EpAnn GrhsAnn -> State [LEpaComment] (EpAnn GrhsAnn)
@@ -306,7 +300,7 @@ workInComments ocs new = cs'
                                         (sortEpaComments $ fc ++ cs_after)
              where
                (cs_before,cs_after)
-                   = break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos $ anchor ac) )
+                   = break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos $ epaLocationRealSrcSpan ac) )
                            new
 
 insertTopLevelCppComments ::  HsModule GhcPs -> [LEpaComment] -> (HsModule GhcPs, [LEpaComment])
@@ -322,7 +316,7 @@ insertTopLevelCppComments (HsModule (XModulePs an lo mdeprec mbDoc) mmn mexports
         Just _ ->
             -- We have a module name. Capture all comments up to the `where`
             let
-              (these, remaining) = splitOnWhere Before (am_main $ anns an) cs
+              (these, remaining) = splitOnWhere Before (am_where $ anns an) cs
               (EpAnn a anno ocs) = an :: EpAnn AnnsModule
               anm = EpAnn a anno (workInComments ocs these)
             in
@@ -330,15 +324,15 @@ insertTopLevelCppComments (HsModule (XModulePs an lo mdeprec mbDoc) mmn mexports
     (an1,cs0a) = case lo of
         EpExplicitBraces (EpTok (EpaSpan (RealSrcSpan s _))) _close ->
             let
-                (stay,cs0a') = break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos $ s)) cs0
+                (stay,cs0a') = break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos $ s)) cs0
                 cs' = workInComments (comments an0) stay
             in (an0 { comments = cs' }, cs0a')
         _ -> (an0,cs0)
     -- Deal with possible leading semis
     (an2, cs0b) = case am_decls $ anns an1 of
-        (AddSemiAnn (EpaSpan (RealSrcSpan s _)):_) -> (an1 {comments = cs'}, cs0b')
+        (AddSemiAnn (EpTok (EpaSpan (RealSrcSpan s _))):_) -> (an1 {comments = cs'}, cs0b')
           where
-            (stay,cs0b') = break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos $ s)) cs0a
+            (stay,cs0b') = break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos $ s)) cs0a
             cs' = workInComments (comments an1) stay
         _ -> (an1,cs0a)
 
@@ -349,10 +343,10 @@ insertTopLevelCppComments (HsModule (XModulePs an lo mdeprec mbDoc) mmn mexports
                          where
                            hc1' = workInComments (comments an2) csh'
                            an3' = an2 { comments = hc1' }
-                           (csh', cs0b') = case al_open $ anns l of
-                               Just (AddEpAnn _ (EpaSpan (RealSrcSpan s _))) ->(h, n)
+                           (csh', cs0b') = case annListBracketsLocs $ al_brackets $ anns l of
+                               (EpaSpan (RealSrcSpan s _),_) ->(h, n)
                                  where
-                                   (h,n) = break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos s) )
+                                   (h,n) = break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos s) )
                                        cs0b
 
                                _ -> ([], cs0b)
@@ -366,9 +360,9 @@ insertTopLevelCppComments (HsModule (XModulePs an lo mdeprec mbDoc) mmn mexports
     -- Either hc0i or hc0d should have comments. Combine them
     hc0 = hc0i ++ hc0d
 
-    (hc1,hc_cs) = if null ( am_main $ anns an3)
+    (hc1,hc_cs) = if NoEpTok == (am_where $ anns an3)
         then (hc0,[])
-        else splitOnWhere After (am_main $ anns an3)  hc0
+        else splitOnWhere After (am_where $ anns an3)  hc0
     hc2 = workInComments (comments an3) hc1
     an4 = an3 { anns = (anns an3) {am_cs = hc_cs}, comments = hc2 }
 
@@ -384,16 +378,23 @@ insertTopLevelCppComments (HsModule (XModulePs an lo mdeprec mbDoc) mmn mexports
         cs4' = workInComments cs4 these
         (xs',rest') = allocPreceding xs rest
 
+annListBracketsLocs :: AnnListBrackets -> (EpaLocation,EpaLocation)
+annListBracketsLocs (ListParens o c) = (getEpTokenLoc o,    getEpTokenLoc c)
+annListBracketsLocs (ListBraces o c) = (getEpTokenLoc o,    getEpTokenLoc c)
+annListBracketsLocs (ListSquare o c) = (getEpTokenLoc o,    getEpTokenLoc c)
+annListBracketsLocs (ListBanana o c) = (getEpUniTokenLoc o, getEpUniTokenLoc c)
+annListBracketsLocs ListNone         = (noAnn,              noAnn)
+
+
 data SplitWhere = Before | After
-splitOnWhere :: SplitWhere -> [AddEpAnn] -> [LEpaComment] -> ([LEpaComment], [LEpaComment])
-splitOnWhere _ [] csIn = (csIn,[])
-splitOnWhere w (AddEpAnn AnnWhere (EpaSpan (RealSrcSpan s _)):_) csIn = (hc, fc)
+
+splitOnWhere :: SplitWhere -> EpToken "where" -> [LEpaComment] -> ([LEpaComment], [LEpaComment])
+splitOnWhere w (EpTok (EpaSpan (RealSrcSpan s _))) csIn = (hc, fc)
   where
     splitFunc Before anc_pos c_pos = c_pos < anc_pos
     splitFunc After  anc_pos c_pos = anc_pos < c_pos
-    (hc,fc) = break (\(L ll _) ->  splitFunc w (ss2pos $ anchor ll) (ss2pos s)) csIn
-splitOnWhere _ (AddEpAnn AnnWhere _:_) csIn = (csIn, [])
-splitOnWhere f (_:as) csIn = splitOnWhere f as csIn
+    (hc,fc) = break (\(L ll _) ->  splitFunc w (ss2pos $ epaLocationRealSrcSpan ll) (ss2pos s)) csIn
+splitOnWhere _ _ csIn = (csIn,[])
 
 balanceFirstLocatedAComments :: [LocatedA a] -> ([LocatedA a], [LEpaComment])
 balanceFirstLocatedAComments [] = ([],[])
@@ -403,7 +404,7 @@ balanceFirstLocatedAComments ((L (EpAnn anc an csd) a):ds) = (L (EpAnn anc an cs
         EpaSpan (RealSrcSpan s _) -> (csd', hc)
                `debug` ("balanceFirstLocatedAComments: (csd,csd',attached,header)=" ++ showAst (csd,csd',attached,header))
           where
-            (priors, inners) =  break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos s) )
+            (priors, inners) =  break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos s) )
                                        (priorComments csd)
             pcds = priorCommentsDeltas' s priors
             (attached, header) = break (\(d,_c) -> d /= 1) pcds
@@ -419,14 +420,14 @@ priorCommentsDeltas' r cs = go r (reverse cs)
   where
     go :: RealSrcSpan -> [LEpaComment] -> [(Int, LEpaComment)]
     go _   [] = []
-    go _   (la@(L l@(EpaDelta dp _) _):las) = (deltaLine dp, la) : go (anchor l) las
-    go rs' (la@(L l _):las) = deltaComment rs' la : go (anchor l) las
+    go _   (la@(L l@(EpaDelta _ dp _) _):las) = (getDeltaLine dp, la) : go (epaLocationRealSrcSpan l) las
+    go rs' (la@(L l _):las) = deltaComment rs' la : go (epaLocationRealSrcSpan l) las
 
     deltaComment :: RealSrcSpan -> LEpaComment -> (Int, LEpaComment)
     deltaComment rs' (L loc c) = (abs(ll - al), L loc c)
       where
         (al,_) = ss2pos rs'
-        (ll,_) = ss2pos (anchor loc)
+        (ll,_) = ss2pos (epaLocationRealSrcSpan loc)
 
 allocatePriorComments
   :: Pos
@@ -434,7 +435,7 @@ allocatePriorComments
   -> ([LEpaComment], [LEpaComment])
 allocatePriorComments ss_loc comment_q =
   let
-    cmp (L l _) = ss2pos (anchor l) <= ss_loc
+    cmp (L l _) = ss2pos (epaLocationRealSrcSpan l) <= ss_loc
     (newAnns,after) = partition cmp comment_q
   in
     (after, newAnns)
@@ -451,7 +452,7 @@ insertRemainingCppComments (L l p) cs = L l p'
             EpTok (EpaSpan (RealSrcSpan s _)) -> ss2pos s
             _ -> (1,1)
         _ -> (1,1)
-    (new_before, new_after) = break (\(L ll _) -> (ss2pos $ anchor ll) > end_loc ) cs
+    (new_before, new_after) = break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > end_loc ) cs
 
     addTrailingComments end_loc' cur new = epaCommentsBalanced pc' fc'
       where
@@ -462,8 +463,8 @@ insertRemainingCppComments (L l p) cs = L l p'
             (L ac _:_) -> (sortEpaComments $ pc ++ cs_before, sortEpaComments $ fc ++ cs_after)
               where
                (cs_before,cs_after)
-                   = if (ss2pos $ anchor ac) > end_loc'
-                       then break (\(L ll _) -> (ss2pos $ anchor ll) > (ss2pos $ anchor ac) ) new
+                   = if (ss2pos $ epaLocationRealSrcSpan ac) > end_loc'
+                       then break (\(L ll _) -> (ss2pos $ epaLocationRealSrcSpan ll) > (ss2pos $ epaLocationRealSrcSpan ac) ) new
                        else (new_before, new_after)
 
 -- ---------------------------------------------------------------------
@@ -480,7 +481,7 @@ tokComment t@(L lt c) =
     (GHC.EpaComment (EpaDocComment dc) pt) -> hsDocStringComments (noCommentsToEpaLocation lt) pt dc
     _ -> [mkComment (normaliseCommentText (ghcCommentText t)) lt (ac_prior_tok c)]
 
-hsDocStringComments :: Anchor -> RealSrcSpan -> GHC.HsDocString -> [Comment]
+hsDocStringComments :: EpaLocation -> RealSrcSpan -> GHC.HsDocString -> [Comment]
 hsDocStringComments _ pt (MultiLineDocString dec (x :| xs)) =
   let
     decStr = printDecorator dec
@@ -544,7 +545,7 @@ normaliseCommentText (x:xs) = x:normaliseCommentText xs
 
 -- |Must compare without span filenames, for CPP injected comments with fake filename
 cmpComments :: Comment -> Comment -> Ordering
-cmpComments (Comment _ l1 _ _) (Comment _ l2 _ _) = compare (ss2pos $ anchor l1) (ss2pos $ anchor l2)
+cmpComments (Comment _ l1 _ _) (Comment _ l2 _ _) = compare (ss2pos $ epaLocationRealSrcSpan l1) (ss2pos $ epaLocationRealSrcSpan l2)
 
 -- |Sort, comparing without span filenames, for CPP injected comments with fake filename
 sortComments :: [Comment] -> [Comment]
@@ -554,19 +555,16 @@ sortComments cs = sortBy cmpComments cs
 sortEpaComments :: [LEpaComment] -> [LEpaComment]
 sortEpaComments cs = sortBy cmp cs
   where
-    cmp (L l1 _) (L l2 _) = compare (ss2pos $ anchor l1) (ss2pos $ anchor l2)
+    cmp (L l1 _) (L l2 _) = compare (ss2pos $ epaLocationRealSrcSpan l1) (ss2pos $ epaLocationRealSrcSpan l2)
 
 -- | Makes a comment which originates from a specific keyword.
-mkKWComment :: AnnKeywordId -> NoCommentsLocation -> Comment
-mkKWComment kw (EpaSpan (RealSrcSpan ss mb))
-  = Comment (keywordToString kw) (EpaSpan (RealSrcSpan ss mb)) ss (Just kw)
-mkKWComment kw (EpaSpan (UnhelpfulSpan _))
-  = Comment (keywordToString kw) (EpaDelta (SameLine 0) NoComments) placeholderRealSpan (Just kw)
-mkKWComment kw (EpaDelta dp cs)
-  = Comment (keywordToString kw) (EpaDelta dp cs) placeholderRealSpan (Just kw)
+mkKWComment :: String -> NoCommentsLocation -> Comment
+mkKWComment kw (EpaSpan (RealSrcSpan ss mb)) = Comment kw (EpaSpan (RealSrcSpan ss mb)) ss (Just kw)
+mkKWComment kw (EpaSpan (UnhelpfulSpan _))   = Comment kw (EpaDelta noSrcSpan (SameLine 0) NoComments) placeholderRealSpan (Just kw)
+mkKWComment kw (EpaDelta ss dp cs)           = Comment kw (EpaDelta ss dp cs) placeholderRealSpan (Just kw)
 
-sortAnchorLocated :: [GenLocated Anchor a] -> [GenLocated Anchor a]
-sortAnchorLocated = sortBy (compare `on` (anchor . getLoc))
+sortAnchorLocated :: [GenLocated EpaLocation a] -> [GenLocated EpaLocation a]
+sortAnchorLocated = sortBy (compare `on` (epaLocationRealSrcSpan . getLoc))
 
 -- | Calculates the distance from the start of a string to the end of
 -- a string.
@@ -602,25 +600,6 @@ name2String = showPprUnsafe
 
 -- ---------------------------------------------------------------------
 
-trailingAnnLoc :: TrailingAnn -> EpaLocation
-trailingAnnLoc (AddSemiAnn ss)    = ss
-trailingAnnLoc (AddCommaAnn ss)   = ss
-trailingAnnLoc (AddVbarAnn ss)    = ss
-trailingAnnLoc (AddDarrowAnn ss)  = ss
-trailingAnnLoc (AddDarrowUAnn ss) = ss
-
-setTrailingAnnLoc :: TrailingAnn -> EpaLocation -> TrailingAnn
-setTrailingAnnLoc (AddSemiAnn _)    ss = (AddSemiAnn ss)
-setTrailingAnnLoc (AddCommaAnn _)   ss = (AddCommaAnn ss)
-setTrailingAnnLoc (AddVbarAnn _)    ss = (AddVbarAnn ss)
-setTrailingAnnLoc (AddDarrowAnn _)  ss = (AddDarrowAnn ss)
-setTrailingAnnLoc (AddDarrowUAnn _) ss = (AddDarrowUAnn ss)
-
-addEpAnnLoc :: AddEpAnn -> EpaLocation
-addEpAnnLoc (AddEpAnn _ l) = l
-
--- ---------------------------------------------------------------------
-
 type DeclsByTag a = Map.Map DeclTag [(RealSrcSpan, a)]
 
 orderedDecls
@@ -657,7 +636,7 @@ hsDeclsClassDecl dec = case dec of
       decls
           = orderedDecls sortKey $ Map.fromList
               [(ClsSigTag,    map (\(L l s) -> (srs l, L l (SigD noExtField s))) sigs),
-               (ClsMethodTag, map (\(L l s) -> (srs l, L l (ValD noExtField s))) (bagToList methods)),
+               (ClsMethodTag, map (\(L l s) -> (srs l, L l (ValD noExtField s))) methods),
                (ClsAtTag,     map (\(L l s) -> (srs l, L l (TyClD noExtField $ FamDecl noExtField s))) ats),
                (ClsAtdTag,    map (\(L l s) -> (srs l, L l (InstD noExtField $ TyFamInstD noExtField s))) at_defs)
               ]
@@ -681,12 +660,12 @@ partitionWithSortKey
       [LTyFamInstDecl GhcPs], [LDataFamInstDecl GhcPs], [LDocDecl GhcPs])
 partitionWithSortKey = go
   where
-    go [] = ([], emptyBag, [], [], [], [], [])
+    go [] = ([], [], [], [], [], [], [])
     go ((L l decl) : ds) =
       let (tags, bs, ss, ts, tfis, dfis, docs) = go ds in
       case decl of
         ValD _ b
-          -> (ClsMethodTag:tags, L l b `consBag` bs, ss, ts, tfis, dfis, docs)
+          -> (ClsMethodTag:tags, L l b : bs, ss, ts, tfis, dfis, docs)
         SigD _ s
           -> (ClsSigTag:tags, bs, L l s : ss, ts, tfis, dfis, docs)
         TyClD _ (FamDecl _ t)
@@ -724,7 +703,7 @@ hsDeclsLocalBinds :: HsLocalBinds GhcPs -> [LHsDecl GhcPs]
 hsDeclsLocalBinds lb = case lb of
     HsValBinds _ (ValBinds sortKey bs sigs) ->
       let
-        bds = map wrapDecl (bagToList bs)
+        bds = map wrapDecl bs
         sds = map wrapSig sigs
       in
         orderedDeclsBinds sortKey bds sds
@@ -735,7 +714,7 @@ hsDeclsLocalBinds lb = case lb of
 hsDeclsValBinds :: (HsValBindsLR GhcPs GhcPs) -> [LHsDecl GhcPs]
 hsDeclsValBinds (ValBinds sortKey bs sigs) =
       let
-        bds = map wrapDecl (bagToList bs)
+        bds = map wrapDecl bs
         sds = map wrapSig sigs
       in
         orderedDeclsBinds sortKey bds sds
