@@ -109,12 +109,10 @@ noChange _libdir parsed = return parsed
 
 changeBalanceComments :: Changer
 changeBalanceComments _libdir top = do
-  -- let (GHC.L l p) = makeDeltaAst top
   let (GHC.L l p) = top
   let decls0 = GHC.hsmodDecls p
-      (decls,_,w) = runTransform (balanceCommentsList decls0)
+      decls = balanceCommentsList decls0
   let p2 = p { GHC.hsmodDecls = decls}
-  debugM $ "changeBalanceComments:\n" ++ unlines w
   return (GHC.L l p2)
 
 changeMakeDelta :: Changer
@@ -131,7 +129,7 @@ genTest libdir f origFile expectedFile  = do
 
       case res of
         Left m -> return . Left $ ParseFailure (showErrorMessages m)
-        Right (injectedComments, dflags, pmod)  -> do
+        Right (!injectedComments, !dflags, !pmod)  -> do
           (printed', pmod') <- GHC.liftIO (runRoundTrip libdir f pmod injectedComments)
           let useCpp = GHC.xopt LangExt.Cpp dflags
               printed = trimPrinted printed'
@@ -188,7 +186,7 @@ runRoundTrip libdir f !parsedOrig cs = do
   -- putStrLn $ "comments:" ++ showAst cs
   let !parsedOrigWithComments = insertCppComments parsedOrig cs
   -- putStrLn $ "parsedOrigWithComments:" ++ showAst parsedOrigWithComments
-  pmod <- f libdir parsedOrigWithComments
+  !pmod <- f libdir parsedOrigWithComments
   let !printed = exactPrint pmod
   return (printed, pmod)
 
